/*
// File:		ImapResponseHandler.hpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#ifndef __IMAPRESPONSEHANDLER_HPP__
#define __IMAPRESPONSEHANDLER_HPP__

#include <EmaiIMAP.hpp>
#include <vector>

class ImapFolder;

/*!
// @class	ImapResponseHandler
// @brief	ImapResponseHandler class
//
// @see		Emai::ImapResponseHandler
 */
class ImapResponseHandler : public Emai::ImapResponseHandler
{
public:
	typedef std::vector<ImapFolder*>		ImapFolderList;

protected:
	ImapFolderList						mFolders;
	ImapFolderList						mSubscribedFolders;

	/*!
	// @brief	This method is called when server returns STATUS response line
	*/
	virtual void						ProcessStatus(
												const EmaiImapCommandData* inCommandData,
												const EmaiStatusResponse& inResponse);
	virtual void						ProcessCapability(
												const EmaiCapabilityResponse& inResponse);
	virtual void						ProcessList(
												const EmaiListResponse& inResponse);
	virtual void						ProcessLsub(
												const EmaiLsubResponse& inResponse);
	virtual void						ProcessMailboxStatus(
												const EmaiMailboxStatusResponse& inResponse);
	virtual void						ProcessSearch(
												EmaiConstUniCharPtr inFolderName,
												const EmaiSearchResponse& inResponse);
	virtual void						ProcessFlags(
												EmaiConstUniCharPtr inFolderName,
												const EmaiFlagsResponse& inResponse);
	virtual void						ProcessExists(
												EmaiConstUniCharPtr inFolderName,
												const EmaiExistsResponse& inResponse);
	virtual void						ProcessRecent(
												EmaiConstUniCharPtr inFolderName,
												const EmaiRecentResponse& inResponse);
	virtual void						ProcessExpunge(
												EmaiConstUniCharPtr inFolderName,
												const EmaiExpungeResponse& inResponse);
	virtual void						ProcessFetch(
												EmaiConstUniCharPtr inFolderName,
												const EmaiFetchResponse& inResponse);

	static void							ClearList(
												ImapFolderList& inList);
	static ImapFolder*					FindFolder(
												ImapFolderList& inList,
												EmaiConstUniCharPtr inName);

	virtual ImapFolder*					CreateFolderObject(
												const EmaiListResponse& inResponse) const;
public:

	static bool							IsInbox(
												EmaiConstUniCharPtr inName);


										ImapResponseHandler();
	virtual								~ImapResponseHandler();

	void								ClearFolderList();
	void								ClearSubscribedFolderList();
	ImapFolder*							FindFolderByName(
												EmaiConstUniCharPtr inName);
	ImapFolder*							FindSubscribedFolderByName(
												EmaiConstUniCharPtr inName);
	void								DeleteFolderFromFolderList(
												EmaiConstUniCharPtr inName);
	void								RenameFoldersInFolderList(
												EmaiConstUniCharPtr inName,
												EmaiConstUniCharPtr inNewName);
	
	EmaiUint32							GetFoldersCount() const;
	ImapFolder*							GetFolderAtIndex(
												const EmaiUint32 inIndex) const;
	
	EmaiUint32							GetSubscribedFoldersCount() const;
	ImapFolder*							GetSubscribedFolderAtIndex(
												const EmaiUint32 inIndex) const;
};


#endif /*__IMAPRESPONSEHANDLER_HPP__*/
